/*  $RCSfile: dataread.c,v $ */
/*  $Author: nakane $ */
/*  $Date: 1999/12/20 02:56:43 $ */
/*  $Revision: 1.1.1.1 $ */
/*  $State: Exp $ */
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef STDC_HEADERS
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#endif STDC_HEADERS

#include <string.h>

#include "global.h"
#include "map_data.h"

/************************************************************************/
/* If you have to any problems, please change these values.             */
/************************************************************************/
#ifndef MAXCHAR
#define MAXCHAR 512
#endif MAXCHAR



#ifndef  DEFAULT_DATAINIT_FILE 
#define DEFAULT_DATAINIT_FILE "datainit"
#endif

#ifndef  DEFAULT_DUMP_MEM_FILE 
#define DEFAULT_DUMP_MEM_FILE "dumpmem"
#endif   DEFAULT_DUMP_MEM_FILE 

#ifndef SECONDS_MEMSET_FILE 
#define SECONDS_MEMSET_FILE "data_init.sec"
#endif  SECONDS_MEMSET_FILE 

#ifndef SECONDS_MEMDUMP_FILE 
#define MEMDUMP_FILE "data_dump.sec"
#endif  SECONDS_MEMDUMP_FILE 

    
static int bit_slice ( long data, int bit_no );
static FILE* fopen_with_chk(const char *filename, const char *ac_type);
static void output_datainit(const char *mempath, FILE* data_init_fileptr, 
			    FILE* sec_memset_fileptr);
static void output_datadump(const char *mempath, FILE* memdump_fileptr, 
			    FILE* sec_memdump_fileptr);


/* usage: dataread <mem path> <input mem image file> <dump mem image file> */
int main(int argc, char **argv) {
  FILE *data_init_fileptr, *memdump_fileptr;
  FILE *sec_memset_fileptr, *sec_memdump_fileptr;
  char *mem_path;
  int arg_len;

  if ( argc == 4 ) {
    arg_len = strlen(argv[1]);
    mem_path = malloc(sizeof(char)*(arg_len+1));
    if ( !mem_path ) {
      fprintf(stderr, "memory allocation error.\n");
      exit(1);
    }
    strncpy(mem_path, argv[1], arg_len+1);

    data_init_fileptr = fopen_with_chk(argv[2], "r");
    memdump_fileptr  = fopen_with_chk(argv[3], "r");
    sec_memset_fileptr = fopen_with_chk(SECONDS_MEMSET_FILE, "w");
    sec_memdump_fileptr = fopen_with_chk(SECONDS_MEMDUMP_FILE, "w");
  } else {
    fprintf(stderr, 
	    "usage:dataread <memory path> <input mem image file> <dump mem image file>\n");
    exit(1);
  }


  
  output_datainit(mem_path, data_init_fileptr, sec_memset_fileptr);
  output_datadump(mem_path, memdump_fileptr,  sec_memdump_fileptr);

  fclose(data_init_fileptr);
  fclose(memdump_fileptr);
  fclose(sec_memset_fileptr);
  fclose(sec_memdump_fileptr);
  free(mem_path);
  return 0;
}

/* -------------------------- static function --------------------- */
/* ץԻ˽λ fopen */
static FILE* fopen_with_chk(const char *filename, const char *ac_type) {
  FILE* retptr;
  retptr = fopen(filename, ac_type);
  if ( retptr ) {
    return retptr;
  } else {
    fprintf(stderr, "cannot open %s.\n", filename);
    exit(1);
  }
}

/* -------------------------- static function --------------------- */
/* data  bit_no ֤ΥӥåȤ֤. LSB  0 ӥåȤȤ */
static int bit_slice ( long data, int bit_no ) {
  return ( data & (1<<bit_no) ) >> bit_no ;
}

/* -------------------------- static function --------------------- */
/* ǡ SECONDS ץȤϤ.  */
static void output_datainit(const char *mempath, FILE* data_init_fileptr, 
			    FILE* sec_memset_fileptr) {
  char tmpptr[MAXCHAR];
  long  adr,data;
  int j;
  Sfl_mem conv_adr;

  assert( mempath );
  assert( data_init_fileptr );
  assert( sec_memset_fileptr );


  while ( fgets(tmpptr, MAXCHAR, data_init_fileptr) != NULL ) {
    if ( strlen(tmpptr)  < MAXCHAR-1 ) {
      if (tmpptr[0] != '#') {    /* if not comment */
        if ( sscanf(tmpptr, "%lx %lx", &adr, &data) == 2) {
	  for ( j = 0 ; j < 16 ; ++j ) {
	    assert ( !(adr & 0xfffe0000l) );
	    conv_adr = map_data( adr, j ) ;
	    fprintf(sec_memset_fileptr,
		    "memset %s/munit%d/m X%04lx %x\n",
		    mempath,
		    conv_adr.unit_no, conv_adr.adr, bit_slice(data,j));
	  }
	}
      }
    } else {
      fprintf(stderr, "characters in 1 line must be under %d.\n", MAXCHAR);
      exit(1);
    }
  }

}


/* -------------------------- static function --------------------- */
/* פ SECONDS ץȤϤ.  */
static void output_datadump(const char *mempath, FILE* memdump_fileptr, 
			    FILE* sec_memdump_fileptr){
  char tmpptr[MAXCHAR];
  long  adr;
  int j;
  Sfl_mem conv_adr;

  assert( mempath );
  assert( memdump_fileptr );
  assert( sec_memdump_fileptr );

  while ( fgets(tmpptr, MAXCHAR, memdump_fileptr) != NULL ) {
    if ( strlen(tmpptr)  < MAXCHAR-1 ) {
      if (tmpptr[0] != '#') {    /* if not comment */
        if ( sscanf(tmpptr, "%lx", &adr ) == 1) {
        
	  fprintf(sec_memdump_fileptr, "print \"[%04lx] \"\n", adr);
	  for ( j = 0 ; j < 16 ; ++j ) {
	    assert ( !(adr & 0xfffe0000l) );
	    conv_adr = map_data( adr, 15-j ) ;
	    fprintf(sec_memdump_fileptr, "print \"%%B\" %s/munit%d/m@X%04lx\n",
		    mempath, conv_adr.unit_no, conv_adr.adr);
	  }
	  fprintf(sec_memdump_fileptr, "print \"\\n\"\n");
	}
      }
    } else {
      fprintf(stderr, "characters in 1 line must be under %d.\n", MAXCHAR);
      exit(1);
    }
  }


}
